@@ -21,7 +21,7 @@ class AdvancedInterfaceSwitcherMaemo5(StatusMenuItem):
   version = '0.3.7'
   dev_author = 'Kirill Plyashkevich'
   dev_email = 'ru.spirit@gmail.com'
-  dev_years = '2010'
+  dev_years = '2010-2011'
 
   def __init__(self):
 
